# encoding: utf-8

from django.db import models

class ScraperRun(models.Model):
    scraper_label = models.CharField(blank=False, null=False, max_length=20)
    start_time = models.DateTimeField(blank=False, null=False, auto_now=True)
    end_time = models.DateTimeField(blank=True, null=True)
    logs = models.ManyToManyField('okscraper_django.ScraperRunLog')

    def __str__(self):
        start_time = self.start_time.strftime('%d/%m/%y %H:%M')
        status = 'SUCCESS'
        failedLogs = self.logs.exclude(status='INFO')
        if failedLogs.count() > 0:
            status = failedLogs.order_by('-id')[0].status
        return '%s | %s | %s'%(self.scraper_label, start_time, status)

class ScraperRunLog(models.Model):
    time = models.DateTimeField(blank=False, null=False, auto_now=True)
    text = models.TextField(blank=False, null=False)
    status = models.CharField(blank=False, null=False, max_length=10)

    def __str__(self):
        return '%s: %s \n'%(self.status, self.text)


# encoding: utf-8

from django.utils import unittest
from okscraper_django.management.commands.okscrape import Command
from okscraper_django.models import ScraperRun

class ManagementTestCase(unittest.TestCase):

    def test(self):
        command = Command()
        command.handle('okscraper_django.tests', dblog=True)
        self.assertRegexpMatches(r'^okscraper_django.tests | [0-9][0-9]/[0-9][0-9]/[0-9][0-9] [0-9][0-9]:[0-9][0-9] | ERROR$', str(ScraperRun.objects.all()[0]))



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'ScraperRun'
        db.create_table(u'okscraper_django_scraperrun', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('scraper_label', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('start_time', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('end_time', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'okscraper_django', ['ScraperRun'])

        # Adding M2M table for field logs on 'ScraperRun'
        m2m_table_name = db.shorten_name(u'okscraper_django_scraperrun_logs')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('scraperrun', models.ForeignKey(orm[u'okscraper_django.scraperrun'], null=False)),
            ('scraperrunlog', models.ForeignKey(orm[u'okscraper_django.scraperrunlog'], null=False))
        ))
        db.create_unique(m2m_table_name, ['scraperrun_id', 'scraperrunlog_id'])

        # Adding model 'ScraperRunLog'
        db.create_table(u'okscraper_django_scraperrunlog', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('time', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('text', self.gf('django.db.models.fields.TextField')()),
            ('status', self.gf('django.db.models.fields.CharField')(max_length=10)),
        ))
        db.send_create_signal(u'okscraper_django', ['ScraperRunLog'])


    def backwards(self, orm):
        # Deleting model 'ScraperRun'
        db.delete_table(u'okscraper_django_scraperrun')

        # Removing M2M table for field logs on 'ScraperRun'
        db.delete_table(db.shorten_name(u'okscraper_django_scraperrun_logs'))

        # Deleting model 'ScraperRunLog'
        db.delete_table(u'okscraper_django_scraperrunlog')


    models = {
        u'okscraper_django.scraperrun': {
            'Meta': {'object_name': 'ScraperRun'},
            'end_time': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'logs': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['okscraper_django.ScraperRunLog']", 'symmetrical': 'False'}),
            'scraper_label': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'start_time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'okscraper_django.scraperrunlog': {
            'Meta': {'object_name': 'ScraperRunLog'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'text': ('django.db.models.fields.TextField', [], {}),
            'time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['okscraper_django']


# encoding: utf-8

from django.contrib import admin
from models import ScraperRun

class ScraperRunAdmin(admin.ModelAdmin):

    readonly_fields = ScraperRun._meta.get_all_field_names()

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

admin.site.register(ScraperRun, ScraperRunAdmin)



from setuptools import setup, find_packages

setup(
    name='okscraper-django',
    version='0.0.8',
    description='okscraper django integration',
    author='Ori Hoch',
    author_email='ori@uumpa.com',
    license='GPLv3',
    url='https://github.com/orihoch/okscraper-django',
    packages=find_packages(exclude=["tests", "test.*", 'settings', 'manage']),
    install_requires=['okscraper==0.0.4']
)



# encoding: utf-8

from django.core.management.base import BaseCommand
from optparse import make_option
from okscraper_django.models import ScraperRun, ScraperRunLog
from datetime import datetime
import logging

from okscraper.cli.runner import DbLogRunner, LogRunner

class DjangoDbLogRunner(DbLogRunner):

    def post_init(self):
        scraper_label = self._module_name
        if self._scraper_class_name != 'MainScraper':
            scraper_label = '%s %s' % (scraper_label, self._scraper_class_name)
        self._scraperrun = ScraperRun(scraper_label=scraper_label)
        self._scraperrun.save()

    def on_dblog_emit(self, record):
        """@type record: logging.Record"""
        runlog = ScraperRunLog(text=record.getMessage(), status=record.levelname)
        runlog.save()
        self._scraperrun.logs.add(runlog)
        self._scraperrun.save()

    def post_run(self):
        self._scraperrun.end_time = datetime.now()
        self._scraperrun.save()

class Command(BaseCommand):

    args = 'module [class] [arg]..'

    option_list = BaseCommand.option_list + (
        make_option(
            '--dblog', action='store_true', dest="dblog", default=False,
            help='log run details to db for monitoring of scraper jobs'
        ),
    )

    def handle(self, *args, **options):
        runnerArgs = [args[0], args[1] if len(args)>1 else None]
        runnerKwargs = {
            'log_verbosity': options.get('verbosity', '1'),
            'log_handler': logging.StreamHandler()
        }
        if options.get('dblog', False):
            runner = DjangoDbLogRunner(*runnerArgs, **runnerKwargs)
        else:
            runner = LogRunner(*runnerArgs, **runnerKwargs)
        runner.run()
